<?php

namespace Drupal\h5p_challenge\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;
use Drupal\Core\Url;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\CloseModalDialogCommand;
use Drupal\Core\Ajax\RedirectCommand;
use Drupal\h5p_challenge\FetchClass\H5PChallenge;

/**
 * Class H5PChallengeEndForm.
 */
class H5PChallengeEndForm extends FormBase {

  /**
   * Drupal\h5p_challenge\H5PChallengeServiceInterface definition.
   *
   * @var \Drupal\h5p_challenge\H5PChallengeServiceInterface
   */
  protected $h5pChallengeService;

  /**
   * Drupal\Core\Database\Connection definition.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * @var H5PChallenge
   */
  protected $challenge;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->h5pChallengeService = $container->get('h5p.challenge.default');
    $instance->database = $container->get('database');
    return $instance;
  }

  /**
   * Returns the challenge
   *
   * @return H5PChallenge
   *   Challenge object
   */
  public function getChallenge(): H5PChallenge {
    return $this->challenge;
  }

  /**
   * Checks if challenge can be deleted. Only owner is allowed to delete
   * challenges that have already finished and have results sent.
   *
   * @return boolean
   *   TRUE or FALSE
   */
  private function canEnd(): bool {
    $challenge = $this->getChallenge();

    if ((int)$challenge->finished <= time()) {
      return FALSE;
    }

    if ($this->currentUser()->id() === $challenge->user_id) {
      return TRUE;
    }

    return FALSE;
  }

  /**
   * Ends challenge
   */
  private function end(): void {
    $challenge = $this->getChallenge();

    $this->database->update('h5p_challenge')
      ->fields([
        'finished' => time(),
      ])
      ->condition('uuid', $challenge->uuid)
      ->execute();
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'h5p_challenge_end_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $challenge = $this
      ->h5pChallengeService
      ->getChallengeByUuid($this
        ->getRequest()
        ->attributes
        ->get('uuid')
      );

    if (!$challenge) {
      throw new NotFoundHttpException();
    }

    // TODO Consider allowing user with administrator privileges to end any challenge
    if (!$challenge->belongsToUser() || $this->currentUser()->id() !== $challenge->user_id) {
      throw new AccessDeniedHttpException();
    }

    $this->challenge = $challenge;

    $form['body'] = [
      '#type' => 'container',
    ];
    $form['body']['warning'] = [
      '#prefix' => '<strong>',
      '#suffix' => '</strong>',
      '#plain_text' => $this->t('This will end the challenge and can not be undone!'),
    ];

    $form['actions'] = [
      '#type' => 'actions',
    ];
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('End'),
    ];
    $form['actions']['cancel'] = [
      '#type' => 'link',
      '#title' => $this->t('Cancel'),
      '#url' => Url::fromRoute('h5p_challenge.h5p_challenge_controller_results', [
        'uuid' => $challenge->uuid,
      ]),
      '#attributes' => [
        'class' => [
          'button',
        ],
      ],
    ];

    if ($this->getRequest()->isXmlHttpRequest()) {
      $form['actions']['submit']['#ajax'] = [
        'callback' => '::ajaxSubmitForm',
      ];
      $form['actions']['cancel'] = [
        '#type' => 'button',
        '#value' => $this->t('Cancel and close'),
        '#ajax' => [
          'callback' => '::ajaxCancel',
        ],
      ];
    }

    if (!$this->canEnd()) {
      $form['actions']['submit']['#disabled'] = TRUE;
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // This does not do anything as the form is meant for use with AJAX only
  }

  /**
   * AJAX submit action callback.
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   *   AjaxResponse object with dialog close command and redirect command
   *   (redirect command is only made if end actions is possible).
   */
  public function ajaxSubmitForm(): AjaxResponse {
    $challenge = $this->getChallenge();
    $response = new AjaxResponse();

    $response->addCommand(new CloseModalDialogCommand());

    if ($this->canEnd()) {
      $this->end();
      $response->addCommand(new RedirectCommand(Url::fromRoute('h5p_challenge.h5p_challenge_controller_results', [
        'uuid' => $challenge->uuid,
      ])->toString()));
    }

    return $response;
  }

  /**
   * AJAX cancel action callback.
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   *   AjaxResponse object with dialog close command.
   */
  public function ajaxCancel(): AjaxResponse {
    $response = new AjaxResponse();
    $response->addCommand(new CloseModalDialogCommand());

    return $response;
  }

}
